@@ -1,4 +1,4 @@
-/* Copyright (C) 2017-2019 Free Software Foundation, Inc.
+/* Copyright (C) 2017-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,13 +15,19 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef ARCH_AMD64_H
-#define ARCH_AMD64_H
+#ifndef GDB_ARCH_AMD64_H
+#define GDB_ARCH_AMD64_H
 
 #include "gdbsupport/tdesc.h"
 #include <stdint.h>
 
-target_desc *amd64_create_target_description (uint64_t xcr0, bool is_x32,
-					      bool is_linux, bool segments);
+/* Create amd64 target descriptions according to XSTATE_BV.  If
+   IS_X32 is true, create the x32 ones.  If IS_LINUX is true, create
+   target descriptions for Linux.  If SEGMENTS is true, then include
+   the "org.gnu.gdb.i386.segments" feature registers.  */
 
-#endif /* ARCH_AMD64_H */
+target_desc_up amd64_create_target_description (uint64_t xstate_bv,
+						bool is_x32, bool is_linux,
+						bool segments);
+
+#endif /* GDB_ARCH_AMD64_H */
